// VcComClientWrapper.cpp

#include "stdafx.h"
#include "VcComClientWrapper.h"


#include <sstream>
#ifdef _UNICODE
  #define OBCON_STREAM(var, stream) std::wstringstream var;var<<stream;
#else
  #define OBCON_STREAM(var, stream) std::stringstream var;var<<stream;
#endif
#define OBCON_TRACE_INNER(x) OBCON_STREAM(trs,x);OutputDebugString(trs.str().c_str())
#define OBCON_RELEASETRACE(x) {OBCON_TRACE_INNER(x);}
#ifdef _DEBUG
  #define OBCON_DEBUGTRACE(stream) OBCON_RELEASETRACE(stream)
#else
  #define OBCON_DEBUGTRACE(stream) (void(0))
#endif
#ifdef TRACE
  #undef TRACE
#endif
#define TRACE OBCON_DEBUGTRACE
#ifdef RELEASETRACE
  #undef RELEASETRACE
#endif
#define RELEASETRACE OBCON_RELEASETRACE


VcComClientWrapper::VcComClientWrapper() :
    m_callback(NULL)
{
}

VcComClientWrapper::~VcComClientWrapper()
{
}

void VcComClientWrapper::Connect(VcComClientWrapperCallback * callback)
{
    TRACE(_T("VcComClientWrapper::Connect"));

    m_callback = callback;
    HRESULT hres = m_pvc.CreateInstance(__uuidof(VCCOMLib::VcComClient));
    if (hres == S_OK)
    {
        m_pvc->Connect(this);
    }
    else
    {
        //?? failed to get a VcCom.exe client COM object
        RELEASETRACE(_T("VcComClientWrapper::Connect failed to get a VcCom.exe client COM object")
            << _T(" : HRESULT = ") << hres);
    }
}

void VcComClientWrapper::Disconnect()
{
    TRACE(_T("VcComClientWrapper::Disconnect"));

    if (m_pvc)
    {
        try
        {
            m_pvc->Disconnect();
        }
        catch(...)
        {
        }
    }
    m_pvc = NULL;
    m_callback = NULL;
}

void VcComClientWrapper::SendToAll(
        const std::wstring & fromName,
        const std::wstring & fromId,
        const std::wstring & message)
{
    TRACE(_T("VcComClientWrapper::SendToAll"));

    if (m_pvc)
    {
        try
        {
            m_pvc->SendToAll(
                    fromName.c_str(),
                    fromId.c_str(),
                    message.c_str()
                    );
        }
        catch(...)
        {
            //?? failed to call VcCom.exe client COM object
        }
    }
}

void VcComClientWrapper::SendToName(
        const std::wstring & fromName,
        const std::wstring & fromId,
        const std::wstring & toName,
        const std::wstring & message)
{
    TRACE(_T("VcComClientWrapper::SendToName"));

    if (m_pvc)
    {
        try
        {
            m_pvc->SendToName(
                    fromName.c_str(),
                    fromId.c_str(),
                    toName.c_str(),
                    message.c_str()
                    );
        }
        catch(...)
        {
            //?? failed to call VcCom.exe client COM object
        }
    }
}

void VcComClientWrapper::SendToId(
        const std::wstring & fromName,
        const std::wstring & fromId,
        const std::wstring & toId,
        const std::wstring & message)
{
    TRACE(_T("VcComClientWrapper::SendToId"));

    if (m_pvc)
    {
        try
        {
            m_pvc->SendToId(
                    fromName.c_str(),
                    fromId.c_str(),
                    toId.c_str(),
                    message.c_str()
                    );
        }
        catch(...)
        {
            //?? failed to call VcCom.exe client COM object
        }
    }
}

///////////////////////////////////////////////////////////////////
// IVcComClientCallback
// called from the hub via COM

STDMETHODIMP_(ULONG) VcComClientWrapper::AddRef()
{
    return 1;
}

STDMETHODIMP_(ULONG) VcComClientWrapper::Release()
{
    return 1;
}

STDMETHODIMP VcComClientWrapper::QueryInterface(REFIID riid,void __RPC_FAR *__RPC_FAR *ppvObject)
{
	if (InlineIsEqualUnknown(riid))
	{
        // use first interface
		IUnknown* pUnk = (VCCOMLib::IVcComClientCallback*)this;
		pUnk->AddRef();
		*ppvObject = pUnk;
		return S_OK;
	}

	if (InlineIsEqualGUID(__uuidof(VCCOMLib::IVcComClientCallback), riid))
	{
		VCCOMLib::IVcComClientCallback* pUnk = (VCCOMLib::IVcComClientCallback*)this;
		pUnk->AddRef();
		*ppvObject = pUnk;
		return S_OK;
	}

	return E_NOINTERFACE;
}

STDMETHODIMP VcComClientWrapper::raw_SendToAll(
        BSTR fromName,
        BSTR fromId,
        BSTR message)
{
    TRACE(_T("VcComClientWrapper::raw_SendToAll"));

    if (m_callback != NULL)
    {
        m_callback->SendToAll(fromName, fromId, message);
    }
    return S_OK;
}

STDMETHODIMP VcComClientWrapper::raw_SendToName(
        BSTR fromName,
        BSTR fromId,
        BSTR toName,
        BSTR message)
{
    TRACE(_T("VcComClientWrapper::raw_SendToName"));

    if (m_callback != NULL)
    {
        m_callback->SendToName(fromName, fromId, toName, message);
    }
    return S_OK;
}

STDMETHODIMP VcComClientWrapper::raw_SendToId(
        BSTR fromName,
        BSTR fromId,
        BSTR toId,
        BSTR message)
{
    TRACE(_T("VcComClientWrapper::raw_SendToId"));

    if (m_callback != NULL)
    {
        m_callback->SendToId(fromName, fromId, toId, message);
    }
    return S_OK;
}

