//////////////////////////////////////////////////////////////////////
// VcCom
// www.vccom.net
// VcCom API VC++ 6.0 Example Code
// Version 1.00
// Date 01 Aug 2004
/////////////////////////////////////////////////////////////////////
// Description
//
// This file forms part of the VcCom API VC++ 6.0 Example Code
// in order to provide example usage of the VcCom API which is
// available from www.objectconsultans.co.uk
/////////////////////////////////////////////////////////////////////
// How To Use
//
// Please refer to the VcCom API VC++ 6.0 Example Code documentation
/////////////////////////////////////////////////////////////////////

// VcComClientWrapper.cpp

////////////////////////////////////////////////////////////////////
// Precompilied headers 
#include "stdafx.h"	// Comment out this line if you don't use precompilied headers
////////////////////////////////////////////////////////////////////

#include <atlbase.h>
#include "VcComClientWrapper.h"

#include <sstream>
#ifdef _UNICODE
  #define OBCON_STREAM(var, stream) std::wstringstream var;var<<stream;
#else
  #define OBCON_STREAM(var, stream) std::stringstream var;var<<stream;
#endif
#define OBCON_TRACE_INNER(x) OBCON_STREAM(trs,x);OutputDebugString(trs.str().c_str())
#define OBCON_RELEASETRACE(x) {OBCON_TRACE_INNER(x);}
#ifdef _DEBUG
  #define OBCON_DEBUGTRACE(stream) OBCON_RELEASETRACE(stream)
#else
  #define OBCON_DEBUGTRACE(stream) (void(0))
#endif
#ifdef TRACE
  #undef TRACE
#endif
#define TRACE OBCON_DEBUGTRACE
#ifdef RELEASETRACE
  #undef RELEASETRACE
#endif
#define RELEASETRACE OBCON_RELEASETRACE

/////////////////////////////////////////////////////////////////////
// VcComClientWrapper

// Constructor, just set callback to NULL.
VcComClientWrapper::VcComClientWrapper() :
    m_callback(NULL)
{
}

// Destructor
VcComClientWrapper::~VcComClientWrapper()
{
}

// Connect
// This connects your application to the VcCom
// Where
//	callback = the class you have created to receive messages back from the VcCom
void VcComClientWrapper::Connect(VcComClientWrapperCallback * callback)
{
    TRACE(_T("VcComClientWrapper::Connect"));

    m_callback = callback;
    HRESULT hres = m_pvc.CreateInstance(__uuidof(VCCOMLib::VcComClient));
	
    if (hres == S_OK)
    {
        m_pvc->Connect(this);
    }
    else
    {
        // Failed to get a VcCom.exe client COM object
        RELEASETRACE(_T("VcComClientWrapper::Connect failed to get a VcCom.exe client COM object")
            << _T(" : HRESULT = ") << hres);
    }
}

// Connect
// This disconnects your application from the VcCom
void VcComClientWrapper::Disconnect()
{
    TRACE(_T("VcComClientWrapper::Disconnect"));

    if (m_pvc)
    {
        try
        {
            m_pvc->Disconnect();
        }
        catch(...)
        {
            // Failed to call disconnect
		    TRACE(_T("VcComClientWrapper::Disconnect Failed"));
        }
    }
    m_pvc = NULL;
    m_callback = NULL;
}

// Set the name of the application to the passed name.
void VcComClientWrapper::Name( const std::wstring &name )
{
   if (m_pvc)
    {
        try
        {
			m_pvc->SetName( name.c_str() );
        }
        catch(...)
        {
            // Failed to call disconnect
		    TRACE(_T("VcComClientWrapper::Name (Set) Failed"));
        }
    }
}

// Get the current name of the application 
std::wstring VcComClientWrapper::Name( void )
{
	std::wstring currentName;

    if (m_pvc)
    {
        try
        {
			_bstr_t name = m_pvc->GetName();
			currentName = name;
        }
        catch(...)
        {
            // Failed to call disconnect
		    TRACE(_T("VcComClientWrapper::Name (Get) Failed"));
        }
    }
	return currentName;
}

// Get the VcCom ID for this application
std::wstring VcComClientWrapper::Id( void )
{
	std::wstring currentId;

    if (m_pvc)
    {
        try
        {
			_bstr_t theId = m_pvc->GetId();
			currentId = theId;
        }
        catch(...)
        {
            // Failed to call disconnect
		    TRACE(_T("VcComClientWrapper::Name (Get) Failed"));
        }
    }
	return currentId;
}

// SendToAll
// Sends a message to all applications connected to the VcCom
// Where
//	message - the message you want to send
void VcComClientWrapper::SendToAll(const std::wstring & message)
{
    TRACE(_T("VcComClientWrapper::SendToAll"));

    if (m_pvc)
    {
        try
        {
            m_pvc->SendToAll( message.c_str() );
        }
        catch(...)
        {
            // Failed to call VcCom.exe client COM object
		    TRACE(_T("VcComClientWrapper::SendToAll Failed"));
        }
    }
}

// SendToName
// Sends a message to a named application connected to the VcCom
// Where
//	toName - the name of the application you want to send this message to
//	message - the message you want to send
void VcComClientWrapper::SendToName(
        const std::wstring & toName,
        const std::wstring & message)
{
    TRACE(_T("VcComClientWrapper::SendToName"));

    if (m_pvc)
    {
        try
        {
            m_pvc->SendToName(
                    toName.c_str(),
                    message.c_str()
                    );
        }
        catch(...)
        {
            // Failed to call VcCom.exe client COM object
		    TRACE(_T("VcComClientWrapper::SendToName Failed"));
        }
    }
}

// SendToId
// Sends a message to a specified (Id) application connected to the VcCom
// Where
//	toId - the Id of the application you want to send this message to
//	message - the message you want to send
void VcComClientWrapper::SendToId(
        const std::wstring & toId,
        const std::wstring & message)
{
    TRACE(_T("VcComClientWrapper::SendToId"));

    if (m_pvc)
    {
        try
        {
            m_pvc->SendToId(
                    toId.c_str(),
                    message.c_str()
                    );
        }
        catch(...)
        {
            // Failed to call VcCom.exe client COM object
			TRACE(_T("VcComClientWrapper::SendToId Failed"));
        }
    }
}

///////////////////////////////////////////////////////////////////
// IVcComClientCallback
// called from the hub via COM

// AddRef
// Adds a ref count onto object
STDMETHODIMP_(ULONG) VcComClientWrapper::AddRef()
{
    return 1;
}

// Release
// Releases a ref count 
STDMETHODIMP_(ULONG) VcComClientWrapper::Release()
{
    return 1;
}

// QueryInterface
// Admit what interfaces we support
STDMETHODIMP VcComClientWrapper::QueryInterface(REFIID riid,void __RPC_FAR *__RPC_FAR *ppvObject)
{
	if (InlineIsEqualUnknown(riid))
	{
        // use first interface
		IUnknown* pUnk = (VCCOMLib::IVcComClientCallback*)this;
		pUnk->AddRef();
		*ppvObject = pUnk;
		return S_OK;
	}

	if (InlineIsEqualGUID(__uuidof(VCCOMLib::IVcComClientCallback), riid))
	{
		VCCOMLib::IVcComClientCallback* pUnk = (VCCOMLib::IVcComClientCallback*)this;
		pUnk->AddRef();
		*ppvObject = pUnk;
		return S_OK;
	}

	return E_NOINTERFACE;
}

// raw_SendToAll
// Called when a message is sent to all applications
// passes call onto callback wrapper object
STDMETHODIMP VcComClientWrapper::raw_SendToAll(
        BSTR fromName,
        BSTR fromId,
        BSTR message)
{
    TRACE(_T("VcComClientWrapper::raw_SendToAll"));

    if (m_callback != NULL)
    {
        m_callback->SendToAll(fromName, fromId, message);
    }
    return S_OK;
}

// raw_SendToName
// Called when a message is sent to a specific Application Name
// passes call onto callback wrapper object
STDMETHODIMP VcComClientWrapper::raw_SendToName(
        BSTR fromName,
        BSTR fromId,
        BSTR,			// To Name, not passed on as you know your own name
        BSTR message)
{
    TRACE(_T("VcComClientWrapper::raw_SendToName"));

    if (m_callback != NULL)
    {
        m_callback->SendToName(fromName, fromId, message);
    }
    return S_OK;
}

// raw_SendToId
// Called when a message is sent to a specific Application Id
// passes call onto callback wrapper object
STDMETHODIMP VcComClientWrapper::raw_SendToId(
        BSTR fromName,
        BSTR fromId,
        BSTR,			// ToId,not passed on as you know your Id (or you can ask for it)
        BSTR message)
{
    TRACE(_T("VcComClientWrapper::raw_SendToId"));

    if (m_callback != NULL)
    {
        m_callback->SendToId(fromName, fromId, message);
    }
    return S_OK;
}

